function [ results ] = test3_1_rr1l_sfft_algorithm_multipletests( N_array, sparsity_s_array, sparsity_s_local_array, threshold_theta_array, algorithm_name, niter_r, nruns, filename )
%TEST3_1_RR1L_SFFT_ALGORITHM_MULTIPLETESTS Performs tests from Section 4.2.2.
%   A (fixed) 10-dimensional test function with infinitely many non-zero
%   Fourier coefficents is used, which consists of tensor products
%   of several B-splines of different order.
%   For given relative threshold values, approximately the largest
%   Fourier coefficients with corresponding frequencies are reconstructed
%   by Algorithm 3 using Algorithm 4.
%
%   Input parameters:
%      N_array - array of expansion parameters
%      sparsity_s_array - array of sparsity parameters, same length as N_array
%      sparsity_s_local_array - array of internal parameters
%      threshold_theta_array - array of internal parameters
%      algorithm_name - use 'rlfftriabs'
%      niter_r - number of detection iterations
%      nruns - number of repeated test runs
%      filename - output result file name
%
% Copyright (c) 2020 Toni Volkmer
%
% This program is free software; you can redistribute it and/or modify it under
% the terms of the GNU General Public License as published by the Free Software
% Foundation; either version 2 of the License, or (at your option) any later
% version.
%
% This program is distributed in the hope that it will be useful, but WITHOUT
% ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
% FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
% details.
%
% You should have received a copy of the GNU General Public License along with
% this program; if not, write to the Free Software Foundation, Inc., 51
% Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.

if length(N_array) ~= length(sparsity_s_array)
  error('input N_array and sparsity_s_array must be of identical length');
end

if length(N_array) ~= length(sparsity_s_local_array)
  error('input N_array and sparsity_s_local_array must be of identical length');
end

if length(N_array) ~= length(threshold_theta_array)
  error('input N_array and threshold_theta_array must be of identical length');
end

if nargin < 8
  filename = 'test3_1_results.mat';
end

results = repmat(struct(), length(threshold_theta_array), nruns);

for iN=1:length(N_array)
  N = N_array(iN);
  sparsity_s = sparsity_s_array(iN);
  sparsity_s_local = sparsity_s_local_array(iN);
  threshold_theta = threshold_theta_array(iN);
  threshold_theta_basic = 1e-12; %/10;
  for ir = 1:nruns
    [ num_freq_out, err_l2_rel, err_l2_rel_trunc, err_l2_rel_alias, freq_cand_sizes, lattice_sizes, sample_sizes, z, run_times, niter_cg ] ...
      = test3_1_rr1l_sfft_algorithm_singletest( N, algorithm_name, sparsity_s, sparsity_s_local, threshold_theta, threshold_theta_basic, niter_r );
    fprintf('run=%2d, N=%3d, sparsity_s=%d, sparsity_s_local=%d, threshold_theta=%.1e, theta_basic=%.1e, freq_out=%6d, err_l2_rel=%.3e\n', ir, N, sparsity_s, sparsity_s_local, threshold_theta, threshold_theta_basic, num_freq_out, err_l2_rel);

    results(iN,ir).N = N;
    results(iN,ir).sparsity_s = sparsity_s;
    results(iN,ir).sparsity_s_local = sparsity_s_local;
    results(iN,ir).threshold_theta = threshold_theta;
    results(iN,ir).threshold_theta_basic = threshold_theta_basic;
    results(iN,ir).num_freq_out = num_freq_out;
    results(iN,ir).err_l2_rel = err_l2_rel;
    results(iN,ir).err_l2_rel_trunc = err_l2_rel_trunc;
    results(iN,ir).err_l2_rel_alias = err_l2_rel_alias;
    results(iN,ir).freq_cand_sizes = freq_cand_sizes;
    results(iN,ir).lattice_sizes = lattice_sizes;
    results(iN,ir).sample_sizes = sample_sizes;
    results(iN,ir).z = z;
    results(iN,ir).run_times = run_times; % basic, lattice search, sampling, lfft, all
    results(iN,ir).niter_cg = niter_cg;
    save(filename, 'N_array', 'sparsity_s_array', 'sparsity_s_local_array', 'threshold_theta_array', 'algorithm_name', 'niter_r', 'nruns', 'results');
  end
end

end
